#-------------------------------------------------------------------------------
#
#   Mutagenesis - Rendering
#
#-------------------------------------------------------------------------------

from __future__ import division
from math import pi, sin, cos
from pygame import Rect, Surface, draw, transform, SRCALPHA

#from vector_algebra import vector

deg = pi / 180

def vector(x, y):
	return (x, y)

#-------------------------------------------------------------------------------

class Turtle(object):

	mirror = 1
	enable_drawing = True

	def __init__(self, surf, pos, heading):
		self.surface = surf
		self.position = pos
		self.heading = heading
		self.init()
	
	def init(self):
		self.path = []
	
	def clone(self, **kwds):
		cls = self.__class__
		turtle = cls.__new__(cls)
		turtle.__dict__.update(self.__dict__)
		turtle.__dict__.update(kwds)
		turtle.init()
		return turtle
	
	def forward(self, distance, color, width):
		#print "Turtle.forward:", distance, color ###
		a = self.heading * deg
		p0 = self.position
		x0, y0 = p0
		dx = distance * cos(a)
		dy = distance * sin(a)
		x1 = x0 + dx
		y1 = y0 + dy
		p1 = vector(x1, y1)
		#print "...", p0, p1 ###
		if self.enable_drawing:
			draw.line(self.surface, color, p0, p1, width)
		self.position = p1
		self.path.append(p1)
	
	def turn(self, angle):
		self.heading += self.mirror * angle
	
	def turn_for_branch(self, b, branch_angle):
		if b & 1:
			a = - ((b + 1) // 2)
		else:
			a = b // 2
		self.turn(a * branch_angle)

#-------------------------------------------------------------------------------

branch_angle = 20
wood_length = 10
wood_width = 3
wood_color = (0x80, 0x40, 0x00)
fill_color = (0x40, 0xc0, 0x00)
turn_angle = 10

def turn_left(turtle):
	turtle.turn(turn_angle)

def turn_right(turtle):
	turtle.turn(-turn_angle)

def render_wood(turtle):
	turtle.forward(wood_length, wood_color, wood_width)
	
rendering_procs = {
	"l": turn_left,
	"r": turn_right,
	"w": render_wood,
}

#-------------------------------------------------------------------------------

def render_filled_poly(turtle, string, i):
	lturtle = turtle.clone(mirror = 1, enable_drawing = False)
	rturtle = turtle.clone(mirror = -1, enable_drawing = False)
	while 1:
		sym = string[i:i+1]
		i += 1
		if sym == "}" or sym == "":
			break
		proc = rendering_procs.get(sym)
		if proc:
			proc(lturtle)
			proc(rturtle)
	points = [turtle.position] + lturtle.path + list(reversed(rturtle.path))
	draw.polygon(turtle.surface, fill_color, points)
	return i

def render_substring(turtle, string, i):
	#print "render_substring:", turtle, string, i ###
	n = len(string)
	b = 0
	while i < n:
		sym = string[i]
		i += 1
		#print "sym =", sym ###
		if sym == "[":
			b += 1
			branch_turtle = turtle.clone()
			branch_turtle.turn_for_branch(b, branch_angle)
			i = render_substring(branch_turtle, string, i)
		elif sym == "]":
			break
		else:
			b = 0
			if sym == "{":
				i = render_filled_poly(turtle, string, i)
			else:
				proc = rendering_procs.get(sym)
				if proc:
					proc(turtle)
	return i

def render_string(turtle, string):
	render_substring(turtle, string, 0)

def render_plant(plant, size):
	#print "render_plant:", plant, size ###
	buffer = plant.buffer
#	if buffer:
#		w, h = size
#		bw, bh = buffer.get_size()
#		if bw < w or bh < h:
#			buffer = None
	if not buffer:
		size = (600, 600) ###
		#print "render_plant: Rendering", plant, "at", size ###
		buffer = Surface(size, SRCALPHA, 32)
		pos = vector(size[0] // 2, size[1])
		turtle = Turtle(buffer, pos, -90)
		render_string(turtle, plant.structure)
		plant.buffer = buffer
	return buffer

def get_plant_thumbnail(plant, size):
	#print "get_plant_thumbnail:", plant, size ###
	thumb = plant._thumbnail
	if not thumb:
		#print "get_plant_thumbnail: generating thumbnail" ###
		buffer = render_plant(plant, (600, 600))
		thumb = transform.smoothscale(buffer, size)
		plant._thumbnail = thumb
	return thumb
